@@ -23,86 +23,7 @@ import struct
 from datetime import datetime
 from functools import partial
 
-verbose = False
-
-
-def debug(*message):
-    """
-    TODO(wb): replace with logging
-    """
-    global verbose
-    if verbose:
-        print "# [d] %s" % (", ".join(map(str, message)))
-
-
-def warning(message):
-    """
-    TODO(wb): replace with logging
-    """
-    print "# [w] %s" % (message)
-
-
-def info(message):
-    """
-    TODO(wb): replace with logging
-    """
-    print "# [i] %s" % (message)
-
-
-def error(message):
-    """
-    TODO(wb): replace with logging
-    """
-    print "# [e] %s" % (message)
-    sys.exit(-1)
-
-
-def hex_dump(src, start_addr=0):
-    """
-    see:
-    http://code.activestate.com/recipes/142812-hex-dumper/
-    @param src A bytestring containing the data to dump.
-    @param start_addr An integer representing the start
-      address of the data in whatever context it comes from.
-    @return A string containing a classic hex dump with 16
-      bytes per line.  If start_addr is provided, then the
-      data is interpreted as starting at this offset, and
-      the offset column is updated accordingly.
-    """
-    FILTER = ''.join([(len(repr(chr(x))) == 3) and
-                        chr(x) or
-                        '.' for x in range(256)])
-    length = 16
-    result = []
-
-    remainder_start_addr = start_addr
-
-    if start_addr % length != 0:
-        base_addr = start_addr - (start_addr % length)
-        num_spaces = (start_addr % length)
-        num_chars = length - (start_addr % length)
-
-        spaces = " ".join(["  " for i in xrange(num_spaces)])
-        s = src[0:num_chars]
-        hexa = ' '.join(["%02X" % ord(x) for x in s])
-        printable = s.translate(FILTER)
-
-        result.append("%04X   %s %s   %s%s\n" %
-                      (base_addr, spaces, hexa,
-                      " " * (num_spaces + 1), printable))
-
-        src = src[num_chars:]
-        remainder_start_addr = base_addr + length
-
-    for i in xrange(0, len(src), length):
-        s = src[i:i + length]
-        hexa = ' '.join(["%02X" % ord(x) for x in s])
-        printable = s.translate(FILTER)
-        result.append("%04X   %-*s   %s\n" %
-                         (remainder_start_addr + i, length * 3,
-                          hexa, printable))
-
-    return ''.join(result)
+import six
 
 
 class memoize(object):
@@ -138,12 +59,10 @@ class memoize(object):
             cache = obj.__cache
         except AttributeError:
             cache = obj.__cache = {}
-        key = (self.func, args[1:], frozenset(kw.items()))
-        try:
-            res = cache[key]
-        except KeyError:
-            res = cache[key] = self.func(*args, **kw)
-        return res
+        key = (self.func, args[1:], frozenset(list(kw.items())))
+        if key not in cache:
+            cache[key] = self.func(*args, **kw)
+        return cache[key]
 
 
 def align(offset, alignment):
@@ -155,7 +74,7 @@ def align(offset, alignment):
     """
     if offset % alignment == 0:
         return offset
-        return offset + (alignment - (offset % alignment))
+    return offset + (alignment - (offset % alignment))
 
 
 def dosdate(dosdate, dostime):
@@ -186,7 +105,10 @@ def dosdate(dosdate, dostime):
 
 def parse_filetime(qword):
     # see http://integriography.wordpress.com/2010/01/16/using-phython-to-parse-and-present-windows-64-bit-timestamps/
-    return datetime.utcfromtimestamp(float(qword) * 1e-7 - 11644473600)
+    try:
+        return datetime.utcfromtimestamp(float(qword) * 1e-7 - 11644473600)
+    except ValueError:
+        return datetime.min
 
 
 class BinaryParserException(Exception):
@@ -203,10 +125,10 @@ class BinaryParserException(Exception):
         self._value = value
 
     def __repr__(self):
-        return "BinaryParserException(%r)" % (self._value)
+        return "BinaryParserException({!r})".format(self._value)
 
     def __str__(self):
-        return "Binary Parser Exception: %s" % (self._value)
+        return "Binary Parser Exception: {}".format(self._value)
 
 
 class ParseException(BinaryParserException):
@@ -223,23 +145,22 @@ class ParseException(BinaryParserException):
         super(ParseException, self).__init__(value)
 
     def __repr__(self):
-        return "ParseException(%r)" % (self._value)
+        return "ParseException({!r})".format(self._value)
 
     def __str__(self):
-        return "Parse Exception(%s)" % (self._value)
+        return "Parse Exception({})".format(self._value)
 
 
 class OverrunBufferException(ParseException):
     def __init__(self, readOffs, bufLen):
-        tvalue = "read: %s, buffer length: %s" % (hex(readOffs), hex(bufLen))
+        tvalue = "read: {}, buffer length: {}".format(hex(readOffs), hex(bufLen))
         super(ParseException, self).__init__(tvalue)
 
     def __repr__(self):
-        return "OverrunBufferException(%r)" % (self._value)
+        return "OverrunBufferException({!r})".format(self._value)
 
     def __str__(self):
-        return "Tried to parse beyond the end of the file (%s)" % \
-            (self._value)
+        return "Tried to parse beyond the end of the file ({})".format(self._value)
 
 
 class Block(object):
@@ -257,16 +178,12 @@ class Block(object):
         self._buf = buf
         self._offset = offset
         self._implicit_offset = 0
-        #print "-- OBJECT: %s" % self.__class__.__name__
-
+        
     def __repr__(self):
-        return "Block(buf=%r, offset=%r)" % (self._buf, self._offset)
-
-    def __unicode__(self):
-        return u"BLOCK @ %s." % (hex(self.offset()))
+        return "Block(buf={!r}, offset={!r})".format(self._buf, self._offset)
 
     def __str__(self):
-        return str(unicode(self))
+        return str(self)
 
     def declare_field(self, type, name, offset=None, length=None):
         """
@@ -338,7 +255,7 @@ class Block(object):
                                  "for dynamic length strings")
         else:
             raise ParseException("Implicit offset not supported "
-                                 "for type: " + type)
+                                 "for type: {}".format(type))
 
     def current_field_offset(self):
         return self._implicit_offset
@@ -539,10 +456,10 @@ class Block(object):
         - `OverrunBufferException`
         """
         if not length:
-            return ""
+            return bytes("".encode("ascii"))
         o = self._offset + offset
         try:
-            return struct.unpack_from("<%ds" % (length), self._buf, o)[0]
+            return bytes(struct.unpack_from("<{}s".format(length), self._buf, o)[0])
         except struct.error:
             raise OverrunBufferException(o, len(self._buf))
 
@@ -555,7 +472,7 @@ class Block(object):
         Throws:
         - `OverrunBufferException`
         """
-        return self.unpack_binary(offset, length)
+        return self.unpack_binary(offset, length).decode('ascii')
 
     def unpack_wstring(self, offset, length):
         """
@@ -568,11 +485,11 @@ class Block(object):
         - `UnicodeDecodeError`
         """
         try:
-            return self._buf[self._offset + offset:self._offset + offset + \
-                             2 * length].tostring().decode("utf16")
+            return bytes(self._buf[self._offset + offset:self._offset + offset + \
+                                   2 * length]).decode("utf16")
         except AttributeError: # already a 'str' ?
-            return self._buf[self._offset + offset:self._offset + offset + \
-                             2 * length].decode("utf16")
+            return bytes(self._buf[self._offset + offset:self._offset + offset + \
+                                   2 * length]).decode("utf16")
 
     def unpack_dosdate(self, offset):
         """
@@ -631,18 +548,18 @@ class Block(object):
         o = self._offset + offset
 
         try:
-            _bin = self._buf[o:o + 16]
+            _bin = bytes(self._buf[o:o + 16])
         except IndexError:
             raise OverrunBufferException(o, len(self._buf))
 
         # Yeah, this is ugly
-        h = map(ord, _bin)
-        return "%02x%02x%02x%02x-%02x%02x-%02x%02x-%02x%02x-%02x%02x%02x%02x%02x%02x" % \
-            (h[3], h[2], h[1], h[0],
-             h[5], h[4],
-             h[7], h[6],
-             h[8], h[9],
-             h[10], h[11], h[12], h[13], h[14], h[15])
+        h = [six.indexbytes(_bin, i) for i in range(len(_bin))]
+        return """{:02x}{:02x}{:02x}{:02x}-{:02x}{:02x}-{:02x}{:02x}-{:02x}{:02x}-{:02x}{:02x}{:02x}{:02x}{:02x}{:02x}""".format(
+            h[3], h[2], h[1], h[0],
+            h[5], h[4],
+            h[7], h[6],
+            h[8], h[9],
+            h[10], h[11], h[12], h[13], h[14], h[15])
 
     def absolute_offset(self, offset):
         """
