@@ -16,13 +16,15 @@
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
 import re
-import itertools
 import base64
+import itertools
 
-from BinaryParser import Block
-from BinaryParser import hex_dump
-from BinaryParser import ParseException
-from BinaryParser import memoize
+import six
+import hexdump
+
+from .BinaryParser import Block
+from .BinaryParser import ParseException
+from .BinaryParser import memoize
 
 
 class SYSTEM_TOKENS:
@@ -63,6 +65,16 @@ class SuppressConditionalSubstitution(Exception):
         super(SuppressConditionalSubstitution, self).__init__(msg)
 
 
+class UnexpectedStateException(ParseException):
+    """
+    UnexpectedStateException is an exception to be thrown when the parser
+      encounters an unexpected value or state. This probably means there
+      is a bug in the parser, but could stem from a corrupted input file.
+    """
+    def __init__(self, msg):
+        super(UnexpectedStateException, self).__init__(msg)
+
+
 class BXmlNode(Block):
 
     def __init__(self, buf, offset, chunk, parent):
@@ -71,15 +83,15 @@ class BXmlNode(Block):
         self._parent = parent
 
     def __repr__(self):
-        return "BXmlNode(buf=%r, offset=%r, chunk=%r, parent=%r)" % \
-            (self._buf, self.offset(), self._chunk, self._parent)
+        return "BXmlNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
+            self._buf, self.offset(), self._chunk, self._parent)
 
     def __str__(self):
-        return "BXmlNode(offset=%s)" % (hex(self.offset()))
+        return "BXmlNode(offset={})".format(hex(self.offset()))
 
     def dump(self):
-        return hex_dump(self._buf[self.offset():self.offset() + self.length()],
-                        start_addr=self.offset())
+        b = self._buf[self.offset():self.offset() + self.length()]
+        return hexdump.hexdump(b, result='return')
 
     def tag_length(self):
         """
@@ -87,8 +99,7 @@ class BXmlNode(Block):
         @return An integer specifying the length of this tag, not including
           its children.
         """
-        raise NotImplementedError("tag_length not implemented for %r") % \
-            (self)
+        raise NotImplementedError("tag_length not implemented for {!r}").format(self)
 
     def _children(self, max_children=None,
                   end_tokens=[SYSTEM_TOKENS.EndOfStreamToken]):
@@ -99,7 +110,7 @@ class BXmlNode(Block):
         ofs = self.tag_length()
 
         if max_children:
-            gen = xrange(max_children)
+            gen = list(range(max_children))
         else:
             gen = itertools.count()
 
@@ -112,9 +123,9 @@ class BXmlNode(Block):
                 child = HandlerNodeClass(self._buf, self.offset() + ofs,
                                          self._chunk, self)
             except IndexError:
-                raise ParseException("Unexpected token %02X at %s" % \
-                                         (token,
-                                          self.absolute_offset(0x0) + ofs))
+                raise ParseException("Unexpected token {:02X} at {}".format(
+                    token,
+                    self.absolute_offset(0x0) + ofs))
             ret.append(child)
             ofs += child.length()
             if token in end_tokens:
@@ -158,13 +169,13 @@ class NameStringNode(BXmlNode):
         self.declare_field("wstring", "string", length=self.string_length())
 
     def __repr__(self):
-        return "NameStringNode(buf=%r, offset=%r, chunk=%r)" % \
-            (self._buf, self.offset(), self._chunk)
+        return "NameStringNode(buf={!r}, offset={!r}, chunk={!r})".format(
+            self._buf, self.offset(), self._chunk)
 
     def __str__(self):
-        return "NameStringNode(offset=%s, length=%s, end=%s)" % \
-            (hex(self.offset()), hex(self.length()),
-             hex(self.offset() + self.length()))
+        return "NameStringNode(offset={}, length={}, end={})".format(
+            hex(self.offset()), hex(self.length()),
+            hex(self.offset() + self.length()))
 
     def string(self):
         return str(self._string())
@@ -186,12 +197,12 @@ class TemplateNode(BXmlNode):
         self.declare_field("dword", "data_length")
 
     def __repr__(self):
-        return "TemplateNode(buf=%r, offset=%r, chunk=%r, parent=%r)" % \
-            (self._buf, self.offset(), self._chunk, self._parent)
+        return "TemplateNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
+            self._buf, self.offset(), self._chunk, self._parent)
 
     def __str__(self):
-        return "TemplateNode(offset=%s, guid=%s, length=%s)" % \
-            (hex(self.offset()), self.guid(), hex(self.length()))
+        return "TemplateNode(offset={}, guid={}, length={})".format(
+            hex(self.offset()), self.guid(), hex(self.length()))
 
     def tag_length(self):
         return 0x18
@@ -211,12 +222,12 @@ class EndOfStreamNode(BXmlNode):
         super(EndOfStreamNode, self).__init__(buf, offset, chunk, parent)
 
     def __repr__(self):
-        return "EndOfStreamNode(buf=%r, offset=%r, chunk=%r, parent=%r)" % \
-            (self._buf, self.offset(), self._chunk, self._parent)
+        return "EndOfStreamNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
+            self._buf, self.offset(), self._chunk, self._parent)
 
     def __str__(self):
-        return "EndOfStreamNode(offset=%s, length=%s, token=%s)" % \
-            (hex(self.offset()), hex(self.length()), 0x00)
+        return "EndOfStreamNode(offset={}, length={}, token={})".format(
+            hex(self.offset()), hex(self.length()), 0x00)
 
     def flags(self):
         return self.token() >> 4
@@ -256,16 +267,16 @@ class OpenStartElementNode(BXmlNode):
             self._tag_length += new_string.length()
 
     def __repr__(self):
-        return "OpenStartElementNode(buf=%r, offset=%r, chunk=%r)" % \
-            (self._buf, self.offset(), self._chunk)
+        return "OpenStartElementNode(buf={!r}, offset={!r}, chunk={!r})".format(
+            self._buf, self.offset(), self._chunk)
 
     def __str__(self):
-        return "OpenStartElementNode(offset=%s, name=%s, length=%s, token=%s, end=%s, taglength=%s, endtag=%s)" % \
-            (hex(self.offset()), self.tag_name(),
-             hex(self.length()), hex(self.token()),
-             hex(self.offset() + self.length()),
-             hex(self.tag_length()),
-             hex(self.offset() + self.tag_length()))
+        return "OpenStartElementNode(offset={}, name={}, length={}, token={}, end={}, taglength={}, endtag={})".format(
+            hex(self.offset()), self.tag_name(),
+            hex(self.length()), hex(self.token()),
+            hex(self.offset() + self.length()),
+            hex(self.tag_length()),
+            hex(self.offset() + self.tag_length()))
 
     @memoize
     def is_empty_node(self):
@@ -305,12 +316,12 @@ class CloseStartElementNode(BXmlNode):
         self.declare_field("byte", "token", 0x0)
 
     def __repr__(self):
-        return "CloseStartElementNode(buf=%r, offset=%r, chunk=%r, parent=%r)" % \
-            (self._buf, self.offset(), self._chunk, self._parent)
+        return "CloseStartElementNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
+            self._buf, self.offset(), self._chunk, self._parent)
 
     def __str__(self):
-        return "CloseStartElementNode(offset=%s, length=%s, token=%s)" % \
-            (hex(self.offset()), hex(self.length()), hex(self.token()))
+        return "CloseStartElementNode(offset={}, length={}, token={})".format(
+            hex(self.offset()), hex(self.length()), hex(self.token()))
 
     def flags(self):
         return self.token() >> 4
@@ -338,12 +349,12 @@ class CloseEmptyElementNode(BXmlNode):
         self.declare_field("byte", "token", 0x0)
 
     def __repr__(self):
-        return "CloseEmptyElementNode(buf=%r, offset=%r, chunk=%r, parent=%r)" % \
-            (self._buf, self.offset(), self._chunk, self._parent)
+        return "CloseEmptyElementNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
+            self._buf, self.offset(), self._chunk, self._parent)
 
     def __str__(self):
-        return "CloseEmptyElementNode(offset=%s, length=%s, token=%s)" % \
-            (hex(self.offset()), hex(self.length()), hex(0x03))
+        return "CloseEmptyElementNode(offset={}, length={}, token={})".format(
+            hex(self.offset()), hex(self.length()), hex(0x03))
 
     def flags(self):
         return self.token() >> 4
@@ -369,12 +380,12 @@ class CloseElementNode(BXmlNode):
         self.declare_field("byte", "token", 0x0)
 
     def __repr__(self):
-        return "CloseElementNode(buf=%r, offset=%r, chunk=%r, parent=%r)" % \
-            (self._buf, self.offset(), self._chunk, self._parent)
+        return "CloseElementNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
+            self._buf, self.offset(), self._chunk, self._parent)
 
     def __str__(self):
-        return "CloseElementNode(offset=%s, length=%s, token=%s)" % \
-            (hex(self.offset()), hex(self.length()), hex(self.token()))
+        return "CloseElementNode(offset={}, length={}, token={})".format(
+            hex(self.offset()), hex(self.length()), hex(self.token()))
 
     def flags(self):
         return self.token() >> 4
@@ -427,7 +438,7 @@ def get_variant_value(buf, offset, chunk, parent, type_, length=None):
     try:
         TypeClass = types[type_]
     except IndexError:
-        raise NotImplementedError("Type %s not implemented" % (type_))
+        raise NotImplementedError("Type {} not implemented".format(type_))
     return TypeClass(buf, offset, chunk, parent, length=length)
 
 
@@ -443,13 +454,13 @@ class ValueNode(BXmlNode):
         self.declare_field("byte", "type")
 
     def __repr__(self):
-        return "ValueNode(buf=%r, offset=%r, chunk=%r, parent=%r)" % \
-            (self._buf, self.offset(), self._chunk, self._parent)
+        return "ValueNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
+            self._buf, self.offset(), self._chunk, self._parent)
 
     def __str__(self):
-        return "ValueNode(offset=%s, length=%s, token=%s, value=%s)" % \
-            (hex(self.offset()), hex(self.length()),
-             hex(self.token()), self.value().string())
+        return "ValueNode(offset={}, length={}, token={}, value={})".format(
+            hex(self.offset()), hex(self.length()),
+            hex(self.token()), self.value().string())
 
     def flags(self):
         return self.token() >> 4
@@ -489,13 +500,13 @@ class AttributeNode(BXmlNode):
             self._name_string_length += new_string.length()
 
     def __repr__(self):
-        return "AttributeNode(buf=%r, offset=%r, chunk=%r, parent=%r)" % \
-            (self._buf, self.offset(), self._chunk, self._parent)
+        return "AttributeNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
+            self._buf, self.offset(), self._chunk, self._parent)
 
     def __str__(self):
-        return "AttributeNode(offset=%s, length=%s, token=%s, name=%s, value=%s)" % \
-            (hex(self.offset()), hex(self.length()), hex(self.token()),
-             self.attribute_name(), self.attribute_value())
+        return "AttributeNode(offset={}, length={}, token={}, name={}, value={})".format(
+            hex(self.offset()), hex(self.length()), hex(self.token()),
+            self.attribute_name(), self.attribute_value())
 
     def flags(self):
         return self.token() >> 4
@@ -538,12 +549,12 @@ class CDataSectionNode(BXmlNode):
         self.declare_field("wstring", "cdata", length=self.string_length() - 2)
 
     def __repr__(self):
-        return "CDataSectionNode(buf=%r, offset=%r, chunk=%r, parent=%r)" % \
-            (self._buf, self.offset(), self._chunk, self._parent)
+        return "CDataSectionNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
+            self._buf, self.offset(), self._chunk, self._parent)
 
     def __str__(self):
-        return "CDataSectionNode(offset=%s, length=%s, token=%s)" % \
-            (hex(self.offset()), hex(self.length()), 0x07)
+        return "CDataSectionNode(offset={}, length={}, token={})".format(
+            hex(self.offset()), hex(self.length()), 0x07)
 
     def flags(self):
         return self.token() >> 4
@@ -565,7 +576,7 @@ class CDataSectionNode(BXmlNode):
 class EntityReferenceNode(BXmlNode):
     """
     The binary XML node for the system token 0x09.
-    
+
     This is an entity reference node.  That is, something that represents
       a non-XML character, eg. & --> &amp;.
 
@@ -584,16 +595,15 @@ class EntityReferenceNode(BXmlNode):
 
 
     def __repr__(self):
-        return "EntityReferenceNode(buf=%r, offset=%r, chunk=%r, parent=%r)" % \
-            (self._buf, self.offset(), self._chunk, self._parent)
+        return "EntityReferenceNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
+            self._buf, self.offset(), self._chunk, self._parent)
 
     def __str__(self):
-        return "EntityReferenceNode(offset=%s, length=%s, token=%s)" % \
-            (hex(self.offset()), hex(self.length()), hex(0x09))
+        return "EntityReferenceNode(offset={}, length={}, token={})".format(
+            hex(self.offset()), hex(self.length()), hex(0x09))
 
     def entity_reference(self):
-        return "&%s;" % \
-            (self._chunk.strings()[self.string_offset()].string())
+        return "&{};".format(self._chunk.strings()[self.string_offset()].string())
 
     def flags(self):
         return self.token() >> 4
@@ -624,16 +634,15 @@ class ProcessingInstructionTargetNode(BXmlNode):
             self._tag_length += new_string.length()
 
     def __repr__(self):
-        return "ProcessingInstructionTargetNode(buf=%r, offset=%r, chunk=%r, parent=%r)" % \
-            (self._buf, self.offset(), self._chunk, self._parent)
+        return "ProcessingInstructionTargetNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
+            self._buf, self.offset(), self._chunk, self._parent)
 
     def __str__(self):
-        return "ProcessingInstructionTargetNode(offset=%s, length=%s, token=%s)" % \
-            (hex(self.offset()), hex(self.length()), hex(0x0A))
+        return "ProcessingInstructionTargetNode(offset={}, length={}, token={})".format(
+            hex(self.offset()), hex(self.length()), hex(0x0A))
 
     def processing_instruction_target(self):
-        return "<?%s" % \
-            (self._chunk.strings()[self.string_offset()].string())
+        return "<?{}".format(self._chunk.strings()[self.string_offset()].string())
 
     def flags(self):
         return self.token() >> 4
@@ -662,21 +671,21 @@ class ProcessingInstructionDataNode(BXmlNode):
             self._string = self.unpack_wstring(0x3, self.string_length())
         else:
             self._string = ""
-        
+
     def __repr__(self):
-        return "ProcessingInstructionDataNode(buf=%r, offset=%r, chunk=%r, parent=%r)" % \
-            (self._buf, self.offset(), self._chunk, self._parent)
+        return "ProcessingInstructionDataNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
+            self._buf, self.offset(), self._chunk, self._parent)
 
     def __str__(self):
-        return "ProcessingInstructionDataNode(offset=%s, length=%s, token=%s)" % \
-            (hex(self.offset()), hex(self.length()), hex(0x0B))
+        return "ProcessingInstructionDataNode(offset={}, length={}, token={})".format(
+            hex(self.offset()), hex(self.length()), hex(0x0B))
 
     def flags(self):
         return self.token() >> 4
 
     def string(self):
         if self.string_length() > 0:
-            return " %s?>" % (self._string)
+            return " {}?>".format(self._string)
         else:
             return "?>"
 
@@ -707,12 +716,12 @@ class TemplateInstanceNode(BXmlNode):
             self._data_length += new_template.length()
 
     def __repr__(self):
-        return "TemplateInstanceNode(buf=%r, offset=%r, chunk=%r, parent=%r)" % \
-            (self._buf, self.offset(), self._chunk, self._parent)
+        return "TemplateInstanceNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
+            self._buf, self.offset(), self._chunk, self._parent)
 
     def __str__(self):
-        return "TemplateInstanceNode(offset=%s, length=%s, token=%s)" % \
-            (hex(self.offset()), hex(self.length()), hex(0x0C))
+        return "TemplateInstanceNode(offset={}, length={}, token={})".format(
+            hex(self.offset()), hex(self.length()), hex(0x0C))
 
     def flags(self):
         return self.token() >> 4
@@ -751,13 +760,13 @@ class NormalSubstitutionNode(BXmlNode):
         self.declare_field("byte", "type")
 
     def __repr__(self):
-        return "NormalSubstitutionNode(buf=%r, offset=%r, chunk=%r, parent=%r)" % \
-            (self._buf, self.offset(), self._chunk, self._parent)
+        return "NormalSubstitutionNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
+            self._buf, self.offset(), self._chunk, self._parent)
 
     def __str__(self):
-        return "NormalSubstitutionNode(offset=%s, length=%s, token=%s, index=%d, type=%d)" % \
-            (hex(self.offset()), hex(self.length()), hex(self.token()),
-             self.index(), self.type())
+        return "NormalSubstitutionNode(offset={}, length={}, token={}, index={}, type={})".format(
+            hex(self.offset()), hex(self.length()), hex(self.token()),
+            self.index(), self.type())
 
     def flags(self):
         return self.token() >> 4
@@ -788,12 +797,12 @@ class ConditionalSubstitutionNode(BXmlNode):
         self.declare_field("byte", "type")
 
     def __repr__(self):
-        return "ConditionalSubstitutionNode(buf=%r, offset=%r, chunk=%r, parent=%r)" % \
-            (self._buf, self.offset(), self._chunk, self._parent)
+        return "ConditionalSubstitutionNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
+            self._buf, self.offset(), self._chunk, self._parent)
 
     def __str__(self):
-        return "ConditionalSubstitutionNode(offset=%s, length=%s, token=%s)" % \
-            (hex(self.offset()), hex(self.length()), hex(0x0E))
+        return "ConditionalSubstitutionNode(offset={}, length={}, token={})".format(
+            hex(self.offset()), hex(self.length()), hex(0x0E))
 
     def should_suppress(self, substitutions):
         sub = substitutions[self.index()]
@@ -829,12 +838,12 @@ class StreamStartNode(BXmlNode):
         self.declare_field("word", "unknown1")
 
     def __repr__(self):
-        return "StreamStartNode(buf=%r, offset=%r, chunk=%r, parent=%r)" % \
-            (self._buf, self.offset(), self._chunk, self._parent)
+        return "StreamStartNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
+            self._buf, self.offset(), self._chunk, self._parent)
 
     def __str__(self):
-        return "StreamStartNode(offset=%s, length=%s, token=%s)" % \
-            (hex(self.offset()), hex(self.length()), hex(self.token()))
+        return "StreamStartNode(offset={}, length={}, token={})".format(
+            hex(self.offset()), hex(self.length()), hex(self.token()))
 
     def verify(self):
         return self.flags() == 0x0 and \
@@ -863,12 +872,12 @@ class RootNode(BXmlNode):
         super(RootNode, self).__init__(buf, offset, chunk, parent)
 
     def __repr__(self):
-        return "RootNode(buf=%r, offset=%r, chunk=%r, parent=%r)" % \
-            (self._buf, self.offset(), self._chunk, self._parent)
+        return "RootNode(buf={!r}, offset={!r}, chunk={!r}, parent={!r})".format(
+            self._buf, self.offset(), self._chunk, self._parent)
 
     def __str__(self):
-        return "RootNode(offset=%s, length=%s)" % \
-            (hex(self.offset()), hex(self.length()))
+        return "RootNode(offset={}, length={})".format(
+            hex(self.offset()), hex(self.length()))
 
     def tag_length(self):
         return 0
@@ -916,7 +925,7 @@ class RootNode(BXmlNode):
         ofs = self.tag_and_children_length()
         sub_count = self.unpack_dword(ofs)
         ofs += 4
-        for _ in xrange(sub_count):
+        for _ in range(sub_count):
             size = self.unpack_word(ofs)
             type_ = self.unpack_byte(ofs + 0x2)
             sub_decl.append((size, type_))
@@ -928,7 +937,7 @@ class RootNode(BXmlNode):
                 sub_def.append(value)
             #[1] = parse_wstring_type_node,
             elif type_ == 0x1:
-                s = self.unpack_wstring(ofs, size / 2).rstrip("\x00")
+                s = self.unpack_wstring(ofs, size // 2).rstrip("\x00")
                 value = s.replace("<", "&gt;").replace(">", "&lt;")
                 sub_def.append(value)
             #[2] = parse_string_type_node,
@@ -976,7 +985,7 @@ class RootNode(BXmlNode):
                 sub_def.append(base64.b64encode(self.unpack_binary(ofs, size)))
             #[15] = parse_guid_type_node,
             elif type_ == 0xF:
-                sub_def.append(self.unpack_guid(ofs))
+                sub_def.append('{' + self.unpack_guid(ofs) + '}')
             #[16] = parse_size_type_node,
             elif type_ == 0x10:
                 if size == 0x4:
@@ -984,7 +993,7 @@ class RootNode(BXmlNode):
                 elif size == 0x8:
                     sub_def.append(self.unpack_qword(ofs))
                 else:
-                    raise "Unexpected size for SizeTypeNode: %s" % hex(size)
+                    raise UnexpectedStateException("Unexpected size for SizeTypeNode: {}".format(hex(size)))
             #[17] = parse_filetime_type_node,
             elif type_ == 0x11:
                 sub_def.append(self.unpack_filetime(ofs))
@@ -997,22 +1006,24 @@ class RootNode(BXmlNode):
                 num_elements = self.unpack_byte(ofs + 1)
                 id_high = self.unpack_dword_be(ofs + 2)
                 id_low = self.unpack_word_be(ofs + 6)
-                value = "S-%d-%d" % (version, (id_high << 16) ^ id_low)
-                for i in xrange(num_elements):
+                value = "S-{}-{}".format(version, (id_high << 16) ^ id_low)
+                for i in range(num_elements):
                     val = self.unpack_dword(ofs + 8 + (4 * i))
-                    value += "-%d" % val
+                    value += "-{}".format(val)
                 sub_def.append(value)
-            #[20] = parse_hex32_type_node,  -- Hex32TypeNoe, 0x14
+            #[20] = parse_hex32_type_node,  -- Hex32TypeNode, 0x14
             elif type_ == 0x14:
                 value = "0x"
-                for c in self.unpack_binary(ofs, size)[::-1]:
-                    value += "%02x" % ord(c)
+                b = self.unpack_binary(ofs, size)[::-1]
+                for i in range(len(b) - 1):
+                    value += '{:02x}'.format(six.indexbytes(b, i))
                 sub_def.append(value)
             #[21] = parse_hex64_type_node,  -- Hex64TypeNode, 0x15
             elif type_ == 0x15:
                 value = "0x"
-                for c in self.unpack_binary(ofs, size)[::-1]:
-                    value += "%02x" % ord(c)
+                b = self.unpack_binary(ofs, size)[::-1]
+                for i in range(len(b) - 1):
+                    value += '{:02x}'.format(six.indexbytes(b, i))
                 sub_def.append(value)
             #[33] = parse_bxml_type_node,  -- BXmlTypeNode, 0x21
             elif type_ == 0x21:
@@ -1023,7 +1034,7 @@ class RootNode(BXmlNode):
                 bin = self.unpack_binary(ofs, size)
                 acc = []
                 while len(bin) > 0:
-                    match = re.search("((?:[^\x00].)+)", bin)
+                    match = re.search(b"((?:[^\x00].)+)", bin)
                     if match:
                         frag = match.group()
                         acc.append("<string>")
@@ -1032,16 +1043,16 @@ class RootNode(BXmlNode):
                         bin = bin[len(frag) + 2:]
                         if len(bin) == 0:
                             break
-                    frag = re.search("(\x00*)", bin).group()
+                    frag = re.search(b"(\x00*)", bin).group()
                     if len(frag) % 2 == 0:
-                        for _ in xrange(len(frag) // 2):
+                        for _ in range(len(frag) // 2):
                             acc.append("<string></string>\n")
                     else:
-                        raise "Error parsing uneven substring of NULLs"
+                        raise ParseException("Error parsing uneven substring of NULLs")
                     bin = bin[len(frag):]
                 sub_def.append("".join(acc))
             else:
-                raise "Unexpected type encountered: %s" % hex(type_)
+                raise "Unexpected type encountered: {}".format(hex(type_))
             ofs += size
         return sub_def
 
@@ -1056,7 +1067,7 @@ class RootNode(BXmlNode):
         ofs = self.tag_and_children_length()
         sub_count = self.unpack_dword(ofs)
         ofs += 4
-        for _ in xrange(sub_count):
+        for _ in range(sub_count):
             size = self.unpack_word(ofs)
             type_ = self.unpack_byte(ofs + 0x2)
             sub_decl.append((size, type_))
@@ -1082,7 +1093,7 @@ class RootNode(BXmlNode):
         sub_count = self.unpack_dword(ofs)
         ofs += 4
         ret = ofs
-        for _ in xrange(sub_count):
+        for _ in range(sub_count):
             size = self.unpack_word(ofs)
             ret += size + 4
             ofs += 4
@@ -1098,18 +1109,17 @@ class VariantTypeNode(BXmlNode):
         self._length = length
 
     def __repr__(self):
-        return "%s(buf=%r, offset=%s, chunk=%r)" % \
-            (self.__class__.__name__, self._buf, hex(self.offset()),
-             self._chunk)
+        return "{}(buf={!r}, offset={}, chunk={!r})".format(
+            self.__class__.__name__, self._buf, hex(self.offset()),
+            self._chunk)
 
     def __str__(self):
-        return "%s(offset=%s, length=%s, string=%s)" % \
-            (self.__class__.__name__, hex(self.offset()),
-             hex(self.length()), self.string())
+        return "{}(offset={}, length={}, string={})".format(
+            self.__class__.__name__, hex(self.offset()),
+            hex(self.length()), self.string())
 
     def tag_length(self):
-        raise NotImplementedError("tag_length not implemented for %r" % \
-                                      (self))
+        raise NotImplementedError("tag_length not implemented for {!r}".format(self))
 
     def length(self):
         return self.tag_length()
@@ -1118,8 +1128,7 @@ class VariantTypeNode(BXmlNode):
         return []
 
     def string(self):
-        raise NotImplementedError("string not implemented for %r" % \
-                                      (self))
+        raise NotImplementedError("string not implemented for {!r}".format(self))
 
 
 class NullTypeNode(object):  # but satisfies the contract of VariantTypeNode, BXmlNode, but not Block
@@ -1163,7 +1172,7 @@ class WstringTypeNode(VariantTypeNode):
                                length=(self.string_length()))
         else:
             self.declare_field("wstring", "_string", 0x0,
-                               length=(self._length / 2))
+                               length=(self._length // 2))
 
     def tag_length(self):
         if self._length is None:
@@ -1416,7 +1425,8 @@ class GuidTypeNode(VariantTypeNode):
         return 16
 
     def string(self):
-        return "{%s}" % (self.guid())
+        print('{' + self.guid() + '}')
+        return '{' + self.guid() + '}'
 
 
 class SizeTypeNode(VariantTypeNode):
@@ -1491,16 +1501,16 @@ class SIDTypeNode(VariantTypeNode):
     @memoize
     def elements(self):
         ret = []
-        for i in xrange(self.num_elements()):
+        for i in range(self.num_elements()):
             ret.append(self.unpack_dword(self.current_field_offset() + 4 * i))
         return ret
 
     @memoize
     def id(self):
-        ret = "S-%d-%d" % \
-            (self.version(), (self.id_high() << 16) ^ self.id_low())
+        ret = "S-{}-{}".format(
+            self.version(), (self.id_high() << 16) ^ self.id_low())
         for elem in self.elements():
-            ret += "-%d" % (elem)
+            ret += "-{}".format(elem)
         return ret
 
     def tag_length(self):
@@ -1525,7 +1535,7 @@ class Hex32TypeNode(VariantTypeNode):
     def string(self):
         ret = "0x"
         for c in self.hex()[::-1]:
-            ret += "%02x" % (ord(c))
+            ret += "{:02x}".format(ord(c))
         return ret
 
 
@@ -1544,7 +1554,7 @@ class Hex64TypeNode(VariantTypeNode):
     def string(self):
         ret = "0x"
         for c in self.hex()[::-1]:
-            ret += "%02x" % (ord(c))
+            ret += "{:02x}".format(c)
         return ret
 
 
@@ -1602,10 +1612,10 @@ class WstringArrayTypeNode(VariantTypeNode):
                     break
             frag = re.search("(\x00*)", bin).group()
             if len(frag) % 2 == 0:
-                for _ in xrange(len(frag) // 2):
+                for _ in range(len(frag) // 2):
                     acc.append("<string></string>\n")
             else:
-                raise "Error parsing uneven substring of NULLs"
+                raise ParseException("Error parsing uneven substring of NULLs")
             bin = bin[len(frag):]
         return "".join(acc)
 
