@@ -26,7 +26,7 @@ def main():
                 continue
             if args.eid != int(get_child(get_child(node, "System"), "EventID").text):
                 continue
-            print record.record_num()
+            print(record.record_num())
 
 
 if __name__ == "__main__":
